% Based on Auerbach and Gorodnichenko (2012) but modified for the
% Estimation of our Bayesian ST-VAR model 


clear;
clc;

randn('seed',1234567)   % set the seed for random generator
rand('seed' ,1234567)   % set the seed for random generator

theta000   = -3;
VARlags000 = 3;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           		PRELIMINARY STEPS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------------------------------------------------------------
%           Sample
%---------------------------------------------------------------------
s_desc.startS   = 56; 
s_desc.endS     = 252;
s_desc.sample0  = s_desc.startS-1:1:s_desc.endS; 
s_desc.sample1  = s_desc.startS:1:s_desc.endS;

% Load data with NBER dates
NBER_dates0

%---------------------------------------------------------------------
%            Conventions
%---------------------------------------------------------------------
% Regimes
%  region 1 = recession
%  region 0 = expansion

%---------------------------------------------------------------------
%            Parameters governing the model
%---------------------------------------------------------------------
% VAR parameters
s_desc.VARlags  = VARlags000;      % number of lags in the VAR
s_desc.irf_hor  = 20;              % number of periods in the impulse response
s_desc.constant = 0;
s_desc.Gpos     = 1;               % position of government spending in the vector of variables in the VAR

% Regime specific intercept and trend
s_exo.trendON=0;      % 1 = include a time trend in the VAR;
% 0 = do not include time trend in the VAR

s_exo.trend_regspec=0;  % 1 = allow regime specific time trend
% 0 = do not allow regime specific time trend

s_exo.inter_regspec=0;  % 1 = allow regime specific intercept
% 0 = do not allow regime specific intercept

%---------------------------------------------------------------------
%                   Bayesian VAR parameters
%---------------------------------------------------------------------

s_prior.flagON=0;   % 1 = use a prior in estimation of VAR slopes
% 0 = do not use a prior in estimation of VAR slopes

s_prior.flagOmegaON=0;
% 1 = use a prior in estimation in cov matrix of VAR residuals
% 0 = do not use a prior in cov matrix of VAR residuals

s_prior.flagOLS=0;
% 1 = use OLS estimates of the VAR as a prior
% 0 = use Minnesota prior for the VAR (note that MN
% prior does not have a prior for the covariance matrix
% of the residuals)

% penalty for theta which governs the curvature in the transition
% probability function
s_prior.theta0              = theta000;    % the slope in the switching equation
s_prior.scale_penalty       = 1e6;
s_prior.scale_penalty_beta  = 0;

% Parameters of the Minnesota prior: random walk for each variable in the VAR
s_prior.lambda_prior=0.5;   % controls cross-variable responses (smaller values mean stronger prior
s_prior.theta_prior=0.5;  % controls own response of variables

%---------------------------------------------------------------------
%                       MCMC step
%---------------------------------------------------------------------

s_MCMC.runMCMCflag=1; % = 1 use MCMC in estimation

s_MCMC.MCMCn=10000;  %%% Sebastian %%%  %100000; %%% Original %%%    % number of MCMC draws
s_MCMC.dropfirstT=min(round(s_MCMC.MCMCn*0.3),20000); % burn-in period for MCMC

%---------------------------------------------------------------------
%           			Import data
%---------------------------------------------------------------------
%% A&G Data
% actual macroeconomic time series: output in data_macro
% data_macro_series %output series
data_macro = importdata('data_macro_series.txt');   

% actual series for government revenue and spending: output in data0
 data_actual_series 
 %Ramey's News Shocks
 data_ramey_0
% switching series: output in data_SV
 data_SV = importdata('data_switching_series.txt'); 

Y = [data_ramey(s_desc.sample0,5)/100 data0(s_desc.sample0,5) data0(s_desc.sample0,6) data_macro(s_desc.sample0,5)  ... 
    data_macro(s_desc.sample0,13) data_macro(s_desc.sample0,12) data_macro(s_desc.sample0,18)];

names = {'Rameys News Shocks'; 'Government Spending'; 'Taxes'; 'Output';  'FFR'; 'Prices'; 'Government Deficit'}; 
s_desc.Nvar=size(Y,2);                  

Npos   = 1;
Gpos   = 2;
Tpos   = 3;
Ypos   = 4;
FFRpos = 5;
Ppos   = 6;
Dpos   = 7;
Shock  = Npos;

GYsc=mean(exp(Y(:,Gpos))./exp(Y(:,Ypos))); %G/Y

tcode = [1 2 2 2 1 2 9];

for i_x = 1:s_desc.Nvar % Transform "X"
     Y(:,i_x) = transx(Y(:,i_x),tcode(i_x)); 
end

Y = Y(2:end,:); %Correct for Transformation

%% Estimation
SV    = 5;
Z     = data_SV(s_desc.sample1-1,SV);
Z0    = (Z-mean(Z))/std(Z);
Z1    = (Z-mean(Z))/std(Z);
theta = 1;

%AG: vector of exogenous variables: lagged values of the switching variable
s_exo.FX=[data_SV(s_desc.sample1-1,SV) data_SV(s_desc.sample1-2,SV) data_SV(s_desc.sample1-3,SV) data_SV(s_desc.sample1-4,SV)];
s_exo.FX=s_exo.FX(s_desc.VARlags+1:end,:);
s_exo.Nforecs=length(s_exo.FX(1,:));


% create the set of basic regressors
X=[];
for i=1:s_desc.VARlags
    X=[X Y(s_desc.VARlags+1-i:end-i,1:s_desc.Nvar)];
end

[rX cX]=size(X);
Y0=Y(s_desc.VARlags+1:end,1:s_desc.Nvar); %Take account for the lags

Z  = Z(s_desc.VARlags+1:end,:);
Z0 = Z0(s_desc.VARlags+1:end,:);

Xm=[X X.*kron(Z0,ones(1,cX)) X.*kron(Z0.^2.*sign(Z0), ones(1,cX)) ones(length(Y0),1) s_exo.FX]; 

s_desc.T=length(Y0);    % sample size
s_data.Y0=Y0;
s_data.X=X;             % just the lags
s_data.Z0=Z0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           	run VARs for initial values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------------------------------------------------------------
%            non-linear VAR to generate residuals
%---------------------------------------------------------------------
% estimate parameters
beta=inv(Xm'*Xm)*(Xm'*Y0);
% compute residuals
resids=Y0-Xm*beta;
% compute the covariance matrix of residuals
Omega=cov(resids);
beta_var=kron(Omega,inv(Xm'*Xm)); %VCV of the beta coefficients

s_lag.AICnonlin=log(det(Omega))+2/length(resids)*length(beta);
s_lag.BICnonlin=log(det(Omega))+log(length(resids))/length(resids)*length(beta);

%---------------------------------------------------------------------
%            linear VAR to generate residuals
%---------------------------------------------------------------------

Xm1=[X ones(length(Y0),1) s_exo.FX]; 
% estimate parameters
betaVARLIN=inv(Xm1'*Xm1)*(Xm1'*Y0);
residsVARLIN=Y0-Xm1*betaVARLIN;
% compute the covariance matrix of residuals
OmegaVARLIN=cov(residsVARLIN);
betaVARLIN=betaVARLIN(1:end-1-s_exo.Nforecs,:);

VARLIN.betaVARLIN=betaVARLIN;
VARLIN.OmegaVARLIN=OmegaVARLIN;
VARLIN.residsVARLIN=residsVARLIN;

s_lag.AIClin=log(det(OmegaVARLIN))+2/length(resids)*length(betaVARLIN);
s_lag.BIClin=log(det(OmegaVARLIN))+log(length(resids))/length(resids)*length(betaVARLIN);

disp('Lag length criteria')
disp(['    Lag = ' num2str(s_desc.VARlags)])
disp('     Non-linear model')
disp(['         AIC =' num2str(s_lag.AICnonlin)])
disp(['         BIC =' num2str(s_lag.BICnonlin)])
disp('     Linear model')
disp(['         AIC =' num2str(s_lag.AIClin)])

disp(['         BIC =' num2str(s_lag.BIClin)]) 

%---------------------------------------------------------------------
%            generate Minnesota prior
%---------------------------------------------------------------------
if s_prior.flagOLS==0 %generate the prior mean and prior variance for the VAR slope coefficients
    [s_prior.prior_mean,s_prior.prior_var]=genMNprior(s_desc.Nvar,s_desc.VARlags,VARLIN.OmegaVARLIN,s_prior.lambda_prior,s_prior.theta_prior);
    
    Dn=dupmat(s_desc.Nvar);
    Dplus=inv(Dn'*Dn)*Dn';
    OmegaVARLIN_var=2*Dplus*kron(OmegaVARLIN,OmegaVARLIN)*Dplus'/s_desc.T;
    
    s_prior.OmegaVAR=1e6*ones(size(diag(OmegaVARLIN_var)));
    s_prior.Omega=OmegaVARLIN;
else
    disp('Error: use MN prior!')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                           NON-Linear VAR
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%---------------------------------------------------------------------
%  First step:
%  estimate the process for the distribution of the error term
%  Omega in recessions and Omega in expansions
%---------------------------------------------------------------------

Omega0=Omega; %Nonlinear Omega

% take a small departure for one of the regimes to make sure that starting
% values are slightly different
D=randn(length(Omega));
Omega1=Omega+min(eig(Omega))*D*D'; %Omegas need to be different in starting value

% Apply Cholesky decomposition since we need to look only for a lower
% trinagular matrix
Omega0chol=chol(Omega0)';
Omega1chol=chol(Omega1)';
s_desc.Omega_length_cov = length(vech(Omega0chol));

% initial parameter values
param0cov=[vech(Omega0chol);  vech(Omega1chol);  s_prior.theta0]; %s_prior.theta0 = curvature

itermax=1000;
disp('fitting first step...')
options=optimset('MaxFunEvals',itermax,'MaxIter',itermax,'Display','off','TolFun',1e-6,'TolX',1e-6);

%obj_logL=max_LSTVAR_mleScov(param0=param0cov,resids=resids,Z0=Z0,Omega_length=s_desc.Omega_length_cov,T=s_desc.T,Nvar=s_desc.Nvar,theta0=s_prior.theta0,scale_penalty=s_prior.scale_penalty)
[param01cov,mleLcov,flagquasi]=fminunc(@max_LSTVAR_mleScov,param0cov,options,resids,Z0,s_desc.Omega_length_cov,s_desc.T,s_desc.Nvar,s_prior.theta0,s_prior.scale_penalty);
[param01cov,mleLcov,flagquasi]=fminsearch(@max_LSTVAR_mleScov,param01cov,options,resids,Z0,s_desc.Omega_length_cov,s_desc.T,s_desc.Nvar,s_prior.theta0,s_prior.scale_penalty);
disp(['logL = ' num2str(mleLcov)])

% convert vectorized matrices with parameters into appropriate matrices
[Omega0E,Omega1E,thetaE]=vec2matScov(param01cov,s_desc.Omega_length_cov); %These are now the MLE Omegas

% Computes the state-depending MLE Slope Coefficients
[beta0E,beta1E,Omega0E,Omega1E,thetaE,gammaE,residsM,beta_const, beta_other,logL]=...
    vec2matSF_struct(param01cov,s_data,s_desc,s_prior,s_exo,s_MCMC);

%% Store matrices
draws       = 5000;
nburn       = 20000;
Omega_E     = zeros(s_desc.Nvar,s_desc.Nvar,draws);
Omega_R     = zeros(s_desc.Nvar,s_desc.Nvar,draws);
Beta_E      = zeros(s_desc.Nvar*s_desc.Nvar*VARlags000,draws);
Beta_R      = zeros(s_desc.Nvar*s_desc.Nvar*VARlags000,draws);
Gamma       = zeros(draws, 1);
laccprobga  = zeros(draws, 1);
laccprobga2 = zeros(draws, 1);
C           = zeros(draws, 1);

%% Prior for Phi, Omega, gamma and c.
% Preliminaries 
prior   = 2;
Z0      = s_data.Z0;
gamma   = 3; 
c       = 0;
M_Z_t   = exp( -gamma/theta * (Z0(:,1)-c))./(1+exp( -gamma/theta * (Z0(:,1) - c )));  
M_Z_t_1 = exp( -gamma/theta * (Z1(:,1)-c))./(1+exp( -gamma/theta * (Z1(:,1) - c )));
XE      = X.*repmat(1-M_Z_t,1,s_desc.Nvar*VARlags000); XE0 = XE;
XR      = X.*repmat(M_Z_t,1,s_desc.Nvar*VARlags000);   XR0 = XR;
E      = Y0 - XE*beta0E' - XR*beta1E';
EE      = E'*E; 
Sigma   = Y0'*Y0;
TR      = trace(inv(Sigma)*EE);
n       = s_desc.Nvar*s_desc.Nvar*VARlags000;
v0       = 0;
S0       = 0;
MH       = 1;
ST_E     = S0 + (Y0 - XE*beta0E')'*(Y0 - XE*beta0E');  %Reasonable!!! Everything that is not explained by B_E   omega_E*s_desc.T; %
ST_R     = S0 + (Y0 - XR*beta1E')'*(Y0 - XR*beta1E');  %Reasonable!!! Everything that is not explained by B_R   omega_R*s_desc.T; %
omega_E   =  iwishrnd(ST_E,s_desc.T);  %10*eye(s_desc.Nvar,s_desc.Nvar);  %Omega0E; %Omega0
omega_R   =  iwishrnd(ST_R,s_desc.T);  %10*eye(s_desc.Nvar,s_desc.Nvar);   %Omega1E; %Omega1
Update    = 0;
a_gamma = 6; % Any values for the Proposal Density
b_gamma = 2; % Any values for the Proposal Density
count   = 0;
niugamma_prior = 9; %prior df
miugamma_prior = 3; %prior mean from the paper: 1
niugamma       = 2*a_gamma; %proposal density df
miugamma       = a_gamma/b_gamma; %proposal density mean
c_min = quantile(Z0,.10);
c_max = quantile(Z0,.90);
aa=30; %grid search for griddy gibbs sampler of c
dc= ( c_max - c_min )/aa; %Use this if trimming

if prior == 1 % Jeffreys Prior
    Beta0    = zeros(n,1); 
    N0_E     = .005*eye(n); 
    N0_R     = .005*eye(n); 
    B_mat    = [beta1E ; eye((s_desc.Nvar)*(VARlags000-1)) zeros((s_desc.Nvar)*(VARlags000-1),s_desc.Nvar)]; 
    V_E      = inv2(kron(inv2(Omega0E),XE'*XE));   
    mE       = beta0E(:); 
    V_R      = inv2(kron(inv2(Omega1E),XR'*XR)); 
    mR       = beta1E(:); 

elseif prior == 2 %Minnesota Prior
    V_E      = inv2(inv2(diag(s_prior.prior_var(:)))                       + kron( XE'*XE,inv2(Omega0E) ));   
    mE       = V_E*(inv2(diag(s_prior.prior_var(:)))*s_prior.prior_mean(:) + kron( XE'*XE,inv2(Omega0E) )*beta0E(:));
    V_R      = inv2(inv2(diag(s_prior.prior_var(:)))                       + kron( XR'*XR,inv2(Omega1E) ));   
    mR       = V_R*(inv2(diag(s_prior.prior_var(:)))*s_prior.prior_mean(:) + kron( XR'*XR,inv2(Omega1E) )*beta1E(:));
    B_mat    = [beta1E ; eye((s_desc.Nvar)*(VARlags000-1)) zeros((s_desc.Nvar)*(VARlags000-1),s_desc.Nvar)];
end
 
%% Drawing
tic;
disp('Sampling')
count_E=zeros(nburn + draws,1);
count_R=zeros(nburn + draws,1);
Eig_R=zeros(nburn + draws,1);
for i = 1: (nburn + draws)
    if mod(i,50) == 0
        disp(i); toc;
    end  
        
        %% Draw Pi
        beta_E = mE + chol(V_E)'*randn(length(vec(mE)),1);
        B_E = reshape(beta_E,s_desc.Nvar,s_desc.Nvar*VARlags000);
        B_mat(1:s_desc.Nvar,:) = B_E;
        
        count_E(i) = 0;
        while max(abs(eig(B_mat))) > 1.01 
             beta_E = mE + chol(V_E)'*randn(length(vec(mE)),1); 
             B_E = reshape(beta_E,s_desc.Nvar,s_desc.Nvar*VARlags000);
             B_mat(1:s_desc.Nvar,:) = B_E;
             count_E(i) = count_E(i)+1;               
        end

         beta_R = vec(mR) + chol(V_R)'*randn(length(vec(mR)),1);
         B_R = reshape(beta_R,s_desc.Nvar,s_desc.Nvar*VARlags000);
         B_mat(1:s_desc.Nvar,:) = B_R;
        
        count_R(i) = 0;
        while max(abs(eig(B_mat))) > 1.01 
             beta_R = mR + chol(V_R)'*randn(length(vec(mR)),1);
             B_R = reshape(beta_R,s_desc.Nvar,s_desc.Nvar*VARlags000);
             B_mat(1:s_desc.Nvar,:) = B_R;
             count_R(i) = count_R(i)+1; 
        end        
        
        %% Draw c
        c1  = c_min; %set equal to minimum if not trimming
        qq = theta; 
        pc_ = []; 
        c1_ = []; 
        pdfvecc_ = [];
        for ii = 1:aa
            c1=c1+dc;
            M_Z_t = exp( -gamma/theta * (Z0(:,1) - c1)) ./ ( 1+exp( -gamma/theta * ( Z0(:,1) -c1 )));
            XE    = X.*repmat(1-M_Z_t,1,s_desc.Nvar*VARlags000); 
            XR    = X.*repmat(M_Z_t,1,s_desc.Nvar*VARlags000);
            E     = Y0 - XE*B_E' - XR*B_R';
            Sigma = Y0'*Y0;
            treesiginv    = trace(inv(Sigma)*E'*E);
            %pc=exp(-(c1^2)/(2*qq^2)-treesiginv/2); %Conditional posterior distribution of c (Gaussian prior with zero mean and std qq.
            pc=exp(-treesiginv/2); %Uniform prior version
            pc_=[pc_ pc]; %set of w's from ritter and 
            c1_=[c1_ c1]; %store the grid values 
        end

        pdfvecc=(pc_)'; %Posterior conditional marginal density of c 
        pdfvecc_=[pdfvecc_ pdfvecc]; %average these after running to see posterior density of c
        crow=(c1_);
        cumsumc=cumsum(pdfvecc);    
        sumc=sum(pdfvecc);
        cdfvecc=cumsumc/sumc; %%posterior conditional marginal cdf
        rncdfc=unifrnd(0,1);
        gg=find(cdfvecc>rncdfc);
        ggy=gg(1,1);
        c=crow(1,ggy); %Use the first value that is larger than a U(0,1) rv

    
    %% Draw gamma
        gammacan=gamm_rnd(1,1,a_gamma, b_gamma);
    
        lpost1=(0.5* niugamma_prior -1)*log(gamma)-gamma*(0.5* niugamma_prior / miugamma_prior)-0.5*TR; %Posterior density for old gamma
        lpost2=(0.5*niugamma-1)*log(gammacan)-gammacan*(0.5* niugamma / miugamma); %Proposal density for new gamma
    
        M_Z_t = exp( -gammacan/theta * ( Z0(:,1)-c ))./( 1+exp( -gammacan/theta * ( Z0(:,1)-c ))); 
        XE    = X.*repmat(1-M_Z_t,1,s_desc.Nvar*VARlags000); 
        XR    = X.*repmat(M_Z_t,1,s_desc.Nvar*VARlags000);

        E     = Y0 - XE*B_E' - XR*B_R';
        EE    = E'*E; 
        Sigma = Y0'*Y0;
        TR    = trace(inv(Sigma)*E'*E);
    
        lcan1         = (0.5*niugamma_prior-1)*log(gammacan)-gammacan*(0.5*niugamma_prior/ miugamma_prior)-0.5*TR; %Posterior density = gamma prior times likelihood
        lcan2         = (0.5*niugamma-1)*log(gamma)-gamma*(0.5*niugamma/ miugamma); %Proposal density for old gamma
        logranga      = log(rand);
        laccprobga(i) = lcan1 - lpost1 + lcan2 - lpost2; 

        
        if logranga < laccprobga(i)
            gamma = gammacan;
            count=count + 1;
        end
          
        % Posterior Values conditional on the new draw of gamma and c
        M_Z_t = exp( -gamma/theta * ( Z0(:,1)-c ))./( 1+exp(-gamma/theta*( Z0(:,1)-c ) ));  % exp(gamma*Z0(:,1))./(1+exp(gamma*Z0(:,1)));
        XE    = X.*repmat(1-M_Z_t,1,s_desc.Nvar*VARlags000); 
        XR    = X.*repmat(M_Z_t,1,s_desc.Nvar*VARlags000); 
        E     = Y0 - XE*B_E' - XR*B_R';

    %% Omega
    
    if MH == 1
        % Type 1 Proposal Distribution
        ST_E     = S0 + (Y0 - XE* B_E')'*(Y0 - XE* B_E')/3;
        ST_R     = S0 + (Y0 - XR* B_R')'*(Y0 - XR* B_R')/3;
        vT       = v0 + s_desc.T/3;
    
        % Proposal Density 
        omega_E_can = iwishrnd(ST_E,vT);
        omega_R_can = iwishrnd(ST_R,vT);

    
            for t1=1:s_desc.T            
                TRO(t1)  = trace( inv2( M_Z_t(t1)*omega_R + (1-M_Z_t(t1))*omega_E ) *(E(t1,:)'*E(t1,:)) ); 
                det_Sigma(t1) = det( M_Z_t(t1)*omega_R + (1-M_Z_t(t1))*omega_E );            
            end
    
            lpost1 =  -1/2*sum(log(det_Sigma)) + ((vT-v0))/2*log(det(omega_E*omega_R)) -.5*sum(TRO);
            lpost2 =  1/2*trace((ST_E-S0)/omega_E) + 1/2*trace((ST_R-S0)/omega_R); %Proposal density for old omegas
                
            for t2=1:s_desc.T
                TRO(t2)  = trace( inv2( M_Z_t(t2)*omega_R_can+(1-M_Z_t(t2))*omega_E_can ) * (E(t2,:)'*E(t2,:))); 
                det_Sigma(t2)=det( M_Z_t(t2)*omega_R_can + (1-M_Z_t(t2))*omega_E_can );
            end 
        
            lcan1 = -1/2*sum(log(det_Sigma)) + ((vT-v0))/2*log(det(omega_E_can*omega_R_can)) -.5*sum(TRO);
            lcan2 =  1/2*trace((ST_E-S0)/omega_E_can) + 1/2*trace((ST_R-S0)/omega_R_can); %Proposal density for new omegas

                
            logranga   = log(rand);
            laccprobga2(i) = lcan1 + lcan2 - lpost1  - lpost2;      
     
            if logranga < laccprobga2(i)
                omega_R = omega_R_can;
                omega_E = omega_E_can;
                Update = Update + 1;
            end
    else %No Metropolis Hastings Step, accept any candidate draw with probability 1.
            ST_E     = S0 + (Y0 - XE0*beta0E')'*(Y0 - XE0*beta0E');  
            ST_R     = S0 + (Y0 - XR0*beta1E')'*(Y0 - XR0*beta1E');  

           omega_R = iwishrnd(ST_E,vT);
           omega_E = iwishrnd(ST_R,vT);
    end
%% Update 
      if prior == 1
          V_E  = inv2(inv2(N0_E) + kron(inv2(omega_E),XE'*XE));
          mE   = beta0E(:); 
          V_R      = inv2(inv2(N0_R) + kron(inv2(omega_R),XR'*XR));
          mR   = beta1E(:);
      elseif prior == 2
          V_E      = inv2(inv2(diag(s_prior.prior_var(:)))                       + kron( XE'*XE,inv2(omega_E) ));   
          mE       = V_E*(inv2(diag(s_prior.prior_var(:)))*s_prior.prior_mean(:) + kron( XE'*XE,inv2(omega_E) )*beta0E(:));     
          V_R      = inv2(inv2(diag(s_prior.prior_var(:)))                       + kron( XR'*XR,inv2(omega_R) ));   
          mR       = V_R*(inv2(diag(s_prior.prior_var(:)))*s_prior.prior_mean(:) + kron( XR'*XR,inv2(omega_R) )*beta1E(:));     
      end
    
%% Save draws after burn-in phase
    if i > nburn
       Beta_E(:,i-nburn) = beta_E;
       Beta_R(:,i-nburn) = beta_R;
       Omega_E(:,:,i-nburn) = omega_E;
       Omega_R(:,:,i-nburn) = omega_R;    
       Gamma(i-nburn)=gamma;
       C(i-nburn)=c;
    end 
end

Beta_E = reshape(Beta_E,s_desc.Nvar,s_desc.Nvar*VARlags000,draws);
Beta_R = reshape(Beta_R,s_desc.Nvar,s_desc.Nvar*VARlags000,draws);

%% Impulse Response Functions

nhor = 21; 
maximum = 10;

%% Shock Expansion
imp_chol_E          = NaN(s_desc.Nvar,nhor,draws);
multiplier_chol_E   = NaN(draws,nhor);
bigj                = zeros(s_desc.Nvar,s_desc.Nvar*VARlags000);
bigj(1:s_desc.Nvar,1:s_desc.Nvar) = eye(s_desc.Nvar);

disp('Shock in Expansion')
for ii = 1:draws
    biga = [Beta_E(:,:,ii) ; eye((s_desc.Nvar)*(VARlags000-1)) zeros((s_desc.Nvar)*(VARlags000-1),s_desc.Nvar)];
    shock = chol(Omega_E(:,:,ii))';
    d = diag(diag(shock));
    shock = inv(d)*shock; %Impact Matrix
    
    impresp = zeros(s_desc.Nvar,s_desc.Nvar*nhor);
    impresp(1:s_desc.Nvar,1:s_desc.Nvar) = shock;
    bigai = biga;
    for j = 1:nhor-1
        impresp(:,j*s_desc.Nvar+1:(j+1)*s_desc.Nvar) = bigj*bigai*bigj'*shock;
        bigai = bigai*biga;       
    end
    impresp    = reshape(impresp,s_desc.Nvar,s_desc.Nvar,nhor);
    imp_chol_E(:,:,ii) = impresp(:,Shock,:);
    
    for jj = 1:s_desc.Nvar
        if tcode(jj) == 2 || tcode(jj) == 5 || tcode(jj) == 9
           imp_chol_E(jj,:,ii) = cumsum(imp_chol_E(jj,:,ii),2); 
        elseif tcode(jj) == 6
            imp_chol_E(jj,:,ii) = cumsum(cumsum(imp_chol_E(jj,:,ii),2),2); 
        end
    end
     
    multiplier_chol_E(ii,:) = multiplier_fct3Vars(imp_chol_E(:,:,ii), GYsc,nhor,Ypos,Gpos);
end

%% Shock Recession
imp_chol_R = NaN(s_desc.Nvar,nhor,draws);
multiplier_chol_R = NaN(draws,nhor);
bigj = zeros(s_desc.Nvar,s_desc.Nvar*VARlags000);
bigj(1:s_desc.Nvar,1:s_desc.Nvar) = eye(s_desc.Nvar);

disp('Shock in Recession')
for ii = 1:draws
    biga = [Beta_R(:,:,ii) ;  eye((s_desc.Nvar)*(VARlags000-1)) zeros((s_desc.Nvar)*(VARlags000-1),s_desc.Nvar)];
    shock = chol(Omega_R(:,:,ii))';
    d = diag(diag(shock));
    shock = inv(d)*shock; %Impact Matrix
    
    impresp = zeros(s_desc.Nvar,s_desc.Nvar*nhor);
    impresp(1:s_desc.Nvar,1:s_desc.Nvar) = shock;
    bigai = biga;
    for j = 1:nhor-1
        impresp(:,j*s_desc.Nvar+1:(j+1)*s_desc.Nvar) = bigj*bigai*bigj'*shock;
        bigai = bigai*biga;
    end
    impresp    = reshape(impresp,s_desc.Nvar,s_desc.Nvar,nhor);
    imp_chol_R(:,:,ii) = impresp(:,Shock,:);
    
    for jj = 1:s_desc.Nvar
        if tcode(jj) == 2 || tcode(jj) == 5 || tcode(jj) == 9 
           imp_chol_R(jj,:,ii) = cumsum(imp_chol_R(jj,:,ii),2); 
        elseif tcode(jj) == 6
            imp_chol_R(jj,:,ii) = cumsum(cumsum(imp_chol_R(jj,:,ii),2),2); 
        end
    end    
    multiplier_chol_R(ii,:) = multiplier_fct3Vars(imp_chol_R(:,:,ii),GYsc,nhor,Ypos,Gpos);
end

%% Save Results

 strA=['Bayesian_TIRFs_backward_4Q_deficit_AG_Ramey_' num2str(abs(s_prior.theta0)) '_' num2str(s_desc.VARlags) '_' num2str(s_desc.startS) '.mat'];
 save(strA)

%% Results
disp('Multiplier')
qus = [16 50, 84]; %16th, 50th and 84th percentile of Multiplier distribution
period = [5 9 21]; %one year, two years and five years after shock
Multiplier_Ramey_E_quant =  squeeze(prctile(multiplier_chol_E(:,period),qus))';
Multiplier_Ramey_R_quant =  squeeze(prctile(multiplier_chol_R(:,period),qus))';
Multiplier_Ramey_quant   = [Multiplier_Ramey_E_quant, Multiplier_Ramey_R_quant];

disp('Cholesky Multipliers')
disp(Multiplier_Ramey_quant) 
  

